import globals
import game
import math
import line
import vector
import copy

light_constant = 0.7		# constant to affect the brightness of the lights
light_cutoff = 0.1	   # cutoff level when the light gives way to darkness

class LightBox:
	"""
	Bounding light box class.
	This defines the limits of our lightsource.
	"""
	def __init__(self, game_map, position, radius):
		"Initiate the box"
		self.setup_size(position, radius)
		self.grab_partial_map(game_map)

	def setup_size(self, position, radius):
		"Work out where the top left and bottom right corners of the box are."
		self.start = position[0] - radius, position[1] - radius
		self.start = max((self.start[0], 0)), max((self.start[1], 0))
		
		self.end   = position[0] + radius, position[1] + radius
		self.end   = ( min((self.end[0], globals.map_size[0])), 
		               min((self.end[1], globals.map_size[1])) )

	def grab_partial_map(self, game_map):
		"Grab a part of the game map delimited by the box."
		self.map = game_map[self.start[1]:self.end[1]]
		for i in range( len(self.map) ):
			self.map[i] = self.map[i][self.start[0]:self.end[0]]

	def reset_illumination(self):
		"Reset the illuminated working values in the box."
		for row in self.map:
			for tile in row:
				tile.illuminated = False

	def edges(self):
		"Return a list of all points upon the edges of the box."
		edges = copy.copy(self.map[0])
		edges.extend(copy.copy(self.map[-1]))
		for row in self.map:
			edges.append(row[0])
			edges.append(row[-1])
		return edges


def distance(a, b):
	"Returns the distance between A and B."
	return vector.magnitude((b[0] - a[0], b[1] - a[1]))


def create_radial(position, amplitude = 1.0):
	"Construct a radial light at a certain position."
	if amplitude < light_cutoff:
		return
	
	radius = find_radius(amplitude)

	# Create bounding box for light; all the light from the light source is
	# contained within this area.
	box = LightBox(game.map, position, radius)
	box.reset_illumination()

	# Get the edges of the box.
	edges = box.edges()

	# Use raytracing to figure out what's illuminated.
	for edge_point in edges:
		light_ray = line.DDA(position, edge_point.position)
		for (x, y) in light_ray:
			tile = game.map[y][x]
			tile.illuminated = True
			if tile.obstacle:
				break
	
	# Work out the amount of light in each square.
	for row in box.map:
		for tile in row:
			if tile.illuminated:
				d = distance(position, tile.position)
				tile.light += radiance(amplitude, d)


def radiance(amplitude, distance):
	"""
	Calculate the radience of a light source at a distance.
	Follows the inverse squared law. A = 1 / (Cd)^2
	"""
	x = float(light_constant * distance)
	a = float(amplitude) / (x * x + 1.0)
	if a < light_cutoff:
		a = 0
	return min((a, 1.0))


def find_radius(amplitude):
	"""
	The radius of a lightsource is the distance at which the strength of the
	light is equal to the cutoff-level.
	d = sqrt( (A - a) / (C^2 * a) )
	"""
	A, a, c = amplitude, light_cutoff, light_constant
	d = math.sqrt((A - a) / (c * c * a))
	return int(d + 1.5)


def blackout():
	"Reset back to darkness."
	for row in game.map:
		for tile in row:
			tile.light = 0.0
