import globals
import error
import game
import misc
import status
import sys

class Mob:
	"Mobile object - ie. a monster or a player."
	def __init__(self, (x, y), image_src):
		self.tile = game.map[y][x]
		self.tile.mob = self
		
		self.image_src = image_src
		self.inventory = []

	def update(self):
		pass

	def move(self, (x, y)):
		"""
		Move the creature to a new location.
		If there's already a creature on that tile, then attack it! Grr!
		"""
		
		if y < 0 or x < 0:
			raise error.InvalidMobMove
		try:
			new_tile = game.map[y][x]
		except IndexError:
			raise error.InvalidMobMove
		if new_tile.mob is not None:
			self.attack(new_tile.mob)
			return
		if new_tile.obstacle:
			raise error.InvalidMobMove

		self.tile.mob = None
		self.tile = new_tile
		new_tile.mob = self
	
	def image(self):
		return self.image_src
		
	def attack(self, target):
		pass


class Creature(Mob):
	"""
	Creatures are mobs with health and attacks.
	Uses simplified D&D fighting and attributes
	"""
	
	def __init__(self, position, image, 
		          name, str, con, level, 
					 hit_dice, movement_cost = 1,
					 weapon = None, armour = None):
		"Initiate the creature."
		Mob.__init__(self, position, image)
		
		# Stats
		self.name = name
		self.str  = str
		self.con  = con
		self.level    = level
		self.hit_dice = hit_dice
		self.hp_array = [misc.dice(*hit_dice) + misc.modifier(con) 
		                 for i in range(level)]
		self.maxhp    = sum(self.hp_array)
		self.hp       = self.maxhp

		# Movement speed
		self.movement_cost = movement_cost
		self.wait = 0
		
		# Equipment
		self.weapon = weapon
		self.armour = armour

		# Other
		self.items  = misc.LimitedList(globals.inventory_limit) 
		self.dead   = False


	def walk(self, (dx, dy)):
		"""
		Walk somewhere.
		(Sometimes creatures have to wait to 'catch their breath' if they are
		slow movers.)
		"""
		self.wait -= (globals.turn - globals.last_turn)
		if self.wait > 0:
			return
		
		position = self.tile.position
		try:
			self.move((position[0] + dx, position[1] + dy))
		except error.InvalidMobMove:
			return
		self.wait = self.movement_cost


	def attack(self, target):
		"Attack something!"
		message = "%s attacks %s!" % (self.name.capitalize(), target.name)
		status.add_message(message)
		try:
			target.wound(self, self.damage())
		except AttributeError:
			return


	def damage(self):
		"Return the amount of damage you do in an attack."
		if self.weapon is None:
			return misc.dice(1, 2) + misc.modifier(self.str)
		else:
			return self.weapon.damage() + misc.modifier(self.str)

	def wound(self, enemy, loss):
		"Something is damaging you."
		self.hp -= loss
		if self.hp <= 0:
			self.die()
			enemy.killed(self)

	def die(self):
		"Die. Become unreferenced, and flag up being dead."
		self.dead = True
		self.tile.mob = None
		self.tile = None

	def killed(self, target):
		"Killed another creature!"
		message = "%s kills %s!" % (self.name.capitalize(), target.name)
		status.add_message(message)

	def add_item(self, item):
		"Add an item to the mob's inventory."
		try:
			self.items.append(item)
		except error.ContainerFull:
			return
		item.container.items.remove(item)
		item.container = self
